<?php
/**
 * CDataFilter class file.
 *
 * @author Seb <serebrov@algo-rithm.com>, Algo-rithm
 *
 * @version 0.1
 *
 * @desc CDataFilter represents information relevant to list (table) data filtration and search.
 *
 * When the data needs to be filtered or searched by some fields, we can use CDataFilter to
 * represent information about filter and search controls
 * These information can be passed to CDataFilterWidget to render
 * filter controls.
 *
 * Note. This class has no relation to yii's CFilter class.
 *
 * Not two filter types are supported:
 *   FILTER_DROPDOWN - simple dropdown list, when option is selected, data filtration
 *                     performed
 *   FILTER_SEARCH - dropdown list with field names and a text filed to enter
 *                   search criteria, when Search button is clicked, search is
 *                   performed
 *
 * Class receives model name, model should support following methods:
 *  - getDataFilterOptions($filterName)
 *  - getDataFilterSearchFields($filterName)
 *  - applyDataFilterCriteria(&$criteria, $filterName, $filterValue)
 *  - applyDataSearchCriteria(&$criteria, $filterName, $searchField, $serachValue)
 *
 * ************************ Using ***********************************
 * *** Controller ***
 * 
 *  public function actionAdmin()
 *	{
 *		$this->processAdminCommand();
 *
 *		$criteria=new CDbCriteria;
 *
 *      $filters = new CDataFilter(User::model());
 *      // add search by user's fields (fields list taken from User::getDataFilterSearchFields())
 *      $filters->addFilter(CDataFilter::FILTER_SEARCH,   'userFieldsSearch', '');
 *      // add dropdown filter by user's school (schools list taken from User::getDataFilterOptions())
 *      $filters->addFilter(CDataFilter::FILTER_DROPDOWN, 'schoolFilter', 'School');
 *      // add dropdown filter by user's class (classes list taken from User::getDataFilterOptions())
 *      $filters->addFilter(CDataFilter::FILTER_DROPDOWN, 'classFilter', 'Class');
 *
 *      $filters->applyCriteria($criteria);
 *
 *      $pages=new CPagination(User::model()->count($criteria));
 *      ...
 *		$sort=new CSort('User');
 *
 *      $models=User::model()->findAll($criteria);
 *
 *      $params = array(
 *		      	'models'=>$models,
 *   			'pages'=>$pages,
 *	      		'sort'=>$sort,
 *              'filters'=>$filters
 *	    );
 *
 *   	if (Yii::app()->request->isAjaxRequest) {
 *          // support for ajax filtering
 *          // "adminList" is parially rendered inside "admin" template
 *          // and contains data table
 *       	$this->renderPartial('adminList', $params);
 *       } else {
 *   		$this->render('admin', $params);
 *       }
 *	}
 *
 * *** Model ***
 *
 * // Filter name is passed
 * // Filter values are available from $_GET array
 * // Method should return array with options for dropdown list
 * // This method only used for FILTER_DROPDOWN filters
 * public function getDataFilterOptions($filterName)
 *   {
 *       switch ($filterName) {
 *           case 'schoolFilter':
 *              $schools = School::model()->findAll();
 *              return CHtml::listData($schools, 'id', 'name');
 *           case 'classFilter':
 *              // This filter is dependent from School - when school is
 *              // selected, only selected School's classes should be shown in dropdown
 *              $schoolId = $_GET['schoolFilter'];
 *              $criteria = new CDbCriteria;
 *              if (isset($schoolId) && $schoolId > 0) {
 *                  $criteria->condition = ' schools_id='.$schoolId.' ';
 *              }
 *              $classes = Classes::model()->findAll($criteria);
 *              return CHtml::listData($classes, 'id', 'name'); 
 *           case 'userTypeFilter':
 *              $userTypes = UserType::model()->findAll();
 *              return CHtml::listData($userTypes, 'id', 'name');
 *       }
 *   }
 *
 *   // Filter name is passed
 *   // Method should return array with search fileds for dropdown list
 *   // This method only used for FILTER_SEARCH filters
 *   public function getDataFilterSearchFields($filterName)
 *   {
 *       switch ($filterName) {
 *           case 'userFieldsSearch':
 *               return array(
 *                   'user_id'=>'User ID',
 *                   'lname'=>'Last Name',
 *                   'fname'=>'First Name',
 *                   'username'=>'User Name',
 *               );
 *       }
 *   }
 *
 *
 *   // Criteria is passed - method should apply condition or other criteria
 *   // parameters to do the actual data filtering
 *   // This method used for FILTER_DROPDOWN and FILTER_SEARCH filters
 *  public function applyDataFilterCriteria(&$criteria, $filterName, $filterValue)
 *   {
 *       if($filterName == 'schoolFilter') {
 *           $localCriteria = new CDbCriteria;
 *           $localCriteria->condition = ' schools_id='.$filterValue.' ';
 *           $criteria->mergeWith($localCriteria);
 *       }
 *
 *       if($filterName == 'classFilter') {
 *           $localCriteria = new CDbCriteria;
 *           ...
 *           $criteria->mergeWith($localCriteria);
 *       }
 *
 *       if ($filterName == 'userTypeFilter') {
 *           $localCriteria = new CDbCriteria;
 *           $localCriteria->condition = ' user_types_id='. $filterValue .' ';
 *           $criteria->mergeWith($localCriteria);
 *       }
 *   }
 *
 *   public function applyDataSearchCriteria(&$criteria, $filterName, $searchField, $searchValue)
 *   {
 *       if($filterName == 'userFieldsSearch') {
 *           $localCriteria = new CDbCriteria;
 *           $localCriteria->condition = ' '.$searchField.' LIKE "%' .$searchValue.'%" ';
 *           $criteria->mergeWith($localCriteria);
 *       }
 *   }
 *
 *  *** View ***
 *  <?php $this->widget('CDataFilterWidget',array('filters'=>$filters)); ?>
 *
 *  OR
 *  // enable ajax filtering / searching - '#updateData' selector will be replaced
 *  <?php $this->widget('CDataFilterWidget',array('filters'=>$filters,
 *        'ajaxMode'=>true, 'updateSelector'=>'#updateData')); ?>
 *  
 */
class CDataFilter extends CComponent
{

    /**
	 * Filter types.
	 */
	const FILTER_DROPDOWN=0;
    const FILTER_SEARCH=1;

    public $filters = array();
    public $model;

	/**
	 * Constructor.
	 * @model to add filter to.
	 */
	public function __construct($model)
	{
        $this->model = $model;
	}   

    /**
     *
     * @param <int> $filterType - FILTER_DROPDOWN or FILTER_SEARCH
     * @param <string> $filterName - filter name
     * @param <type> $filterDisplayName - filter label
     * @param <type> $emptyValue - empty filter value (user only for FILTER_DROPDOWN),
     *               if set to '' - no empty value
     */
	public function addFilter($filterType, $filterName, $filterDisplayName, $emptyValue = "All")
	{
        $this->filters[] = array(
            'type'=>$filterType, 'name'=>$filterName,
            'displayname'=>$filterDisplayName,
            'emptyValue'=>$emptyValue
        );

        if (empty($emptyValue)) {
            $options = $this->model->getDataFilterOptions($filterName);
            $values = array_keys($options);
            list ($value, $name) = each($options);
            // If there is no empty filter value and wepage shown
            // first time (!isset($_GET[$filterName]) - set get to first
            // filter value, to get filtered data
            // The second check - !in_array($_GET[$filterName], $values)
            // is for case, when options list for filter is changed
            // (because of dependency on another filter)
            // then we need to rewrite $_GET[$filterName] to show correct data
            if (!isset($_GET[$filterName]) || !in_array($_GET[$filterName], $values)) {
                $_GET[$filterName] = $value;
            }
        }
	}

    public function applyCriteria(&$criteria)
    {
        foreach ($this->filters as $filter) {
            if ($filter['type'] == CDataFilter::FILTER_DROPDOWN)
                $this->applyCriteriaDropdown($criteria, $filter);
            if ($filter['type'] == CDataFilter::FILTER_SEARCH)
                $this->applyCriteriaSearch($criteria, $filter);
        }
        //$this->model->getDataFilterCondition($criteria);
    }

    public function applyCriteriaDropdown(&$criteria, $filter)
    {
        if(isset($_GET[$filter['name']]) && $_GET[$filter['name']] > 0)
            $this->model->applyDataFilterCriteria($criteria,
                $filter['name'], $_GET[$filter['name']]
            );
    }

    public function applyCriteriaSearch(&$criteria, $filter)
    {
        $textVarName = $filter['name'] . "Text";
        if(isset($_GET[$textVarName]) && !empty($_GET[$textVarName])) {
            
            $searchFields = $this->model->getDataFilterSearchFields($filter['name']);
            $fieldName = $_GET[$filter['name']];
            if ( isset($searchFields[$fieldName])) {
                $this->model->applyDataSearchCriteria(
                        $criteria, $filter['name'],
                        $_GET[$filter['name']], $_GET[$textVarName]
                );
            }
        }        
    }

    /**
     * Returns filters count
     */
    public function getCount()
    {
        return count($this->filters);
    }

}