<?php
/**
 * CDataFilterWidget class file.
 *
 * @author Seb <serebrov@algo-rithm.com>, Algo-rithm
 *
 * @version 0.1
 *
 * @desc CDataFilterWidget renders filter and search controls represented by
 * CDataFilter class.
 *
 */
class CDataFilterWidget extends CWidget
{
    /**
     * @var CDataFilter filters data
     */
    private $_filters;

    /**
     * @var boolean if set to true - ajax filtration / search will be enabled
     */
    private $_ajaxMode = false;
    /**
     *
     * @var string selector name of html element to update by ajax request
     */
    private $_updateSelector = "#updateData";

	/**
	 * @var mixed the CSS file used for the widget. Defaults to null, meaning
	 * using the default CSS file included together with the widget.
	 * If false, no CSS file will be used. Otherwise, the specified CSS file
	 * will be included when using this widget.
	 */
	public $cssFile;

    public function init()
    {
        if($this->cssFile===null)
        {
            $file=dirname(__FILE__).DIRECTORY_SEPARATOR.'datafilter.css';
            $this->cssFile=Yii::app()->getAssetManager()->publish($file);
        }
        parent::init();
    }

    /**
	 * Returns the filters information used by this widget.
	 * @return CDataFilter
	 */
	public function getFilters()
	{		
    	return $this->_filters;
	}

	/**
	 * Sets the filters information used by this widget.
	 * @param CDataFilter
	 */
	public function setFilters($filters)
	{
		$this->_filters = $filters;
	}    

    /**
     *
     * @return <boolean> is ajax mode enabled
     */
    public function getAjaxMode()
    {
        return $this->_ajaxMode;
    }

    /**
     *
     * @param <boolean> $mode - set to true to enable ajax filtration
     */
    public function setAjaxMode($mode)
    {
        $this->_ajaxMode = $mode;
    }

    /**
     *
     * @return <string> selector name - jQuery will update element(s)
     *                  found using this selector after request finished
     */
    public function getUpdateSelector()
    {
        return $this->_updateSelector;
    }

    /**
     *
     * @param <string> $selector - set selector name
     */
    public function setUpdateSelector($selector)
    {
        $this->_updateSelector = $selector;
    }

	/**
	 * Executes the widget.
	 * This overrides the parent implementation by displaying the generated page buttons.
	 */
	public function run()
	{
        //no filters - no widget!
        if ($this->getFilters() != null) {
            $this->buildWidget();
        }
    }

    /**
     * Builds widget
     */
    private function buildWidget()
    {
        $searchFilters = '';
        $dropDownFilters = '';

        foreach ($this->getFilters()->filters as $filter) {            
            if ($filter['type'] == CDataFilter::FILTER_SEARCH)
                $searchFilters .= $this->createFilterSearch($filter);
            if ($filter['type'] == CDataFilter::FILTER_DROPDOWN)
                $dropDownFilters .= $this->createFilterDropDown($filter);
        }

        echo CHtml::form('','get');
        if (!empty($searchFilters)) {
            $this->render('dataFilterSearch', array('searchFilters'=>$searchFilters));            
        }
        if (!empty($dropDownFilters)) {
            $this->render('dataFilterDropdown', array('dropDownFilters'=>$dropDownFilters));
        }
        echo CHtml::endForm();

		$this->registerClientScript();
	}

    /**
     * Creates one dropdown filter
     * @param <array> $filter - filter data
     * @return <string> html with dropdown code
     */
    private function createFilterDropDown($filter)
    {
        $params = $this->_ajaxMode ? array() : array('submit'=>'');
        if (!empty($filter['emptyValue'])) {
            $params['empty'] = $filter['emptyValue'];
        }
        $data = $this->getDataFilterOptions($filter['name']);
        return $this->render('dropdown', array('filter'=>$filter, 
            'data'=>$data, 'params'=>$params), true);
    }

    /**
     * Creates search filter
     * @param <array> $filter - filter data
     * @return <string> html with filter code
     */
    private function createFilterSearch($filter)
    {
        $data = $this->getDataFilterSearchFields($filter['name']);
        return $this->render('search', array('filter'=>$filter, 'data'=>$data), true);        
    }

    /**
     * Returns options for dropdown filter, throws an exception if model returns an empty set
     * @param <string> $filterName
     * @return <string> options for dropdown
     */
    private function getDataFilterOptions($filterName)
    {
        $options = $this->getFilters()->model->getDataFilterOptions($filterName);
        if (!isset($options)) {
            throw new CException('CDataFilterWidget: model\'s method '.
                get_class($this->getFilters()->model) .
                '.getDataFilterOptions() returned empty set.');
        }
        return $options;
    }

    /**
     * Returns seach fields list for search filter dropdown
     * @param <string> $filterName
     * @return <string> search fileds list
     */
    private function getDataFilterSearchFields($filterName)
    {
        $searchFileds = $this->getFilters()->model->getDataFilterSearchFields($filterName);
        if (!isset($searchFileds)) {
            throw new CException('CDataFilterWidget: model\'s method '.
                get_class($this->getFilters()->model) .
                '.getDataFilterSearchFields() returned empty set.');
        }
        return $searchFileds;
    }

	/**
	 * Registers the needed client scripts (mainly CSS file).
	 */
	public function registerClientScript()
	{
		if($this->cssFile!==false)
			self::registerCssFile($this->cssFile);
	}

	/**
	 * Registers the needed CSS file.
	 * @param string the CSS URL. If null, a default CSS URL will be used.
	 * @since 1.0.2
	 */
	public static function registerCssFile($url=null)
	{		
		Yii::app()->getClientScript()->registerCssFile($url);
	}


}